# Generated by Django 2.0.8 on 2018-09-20 00:10

from django.db import migrations, models
import ls.joyous.fields


class Migration(migrations.Migration):

    dependencies = [
        ('joyous', '0009_alter_uid_notnull'),
    ]

    operations = [
        migrations.AddField(
            model_name='calendarpage',
            name='default_view',
            field=models.CharField(choices=[('L', 'List View'), ('W', 'Weekly View'), ('M', 'Monthly View')], default='M', max_length=15),
        ),
        migrations.AddField(
            model_name='calendarpage',
            name='view_choices',
            field=ls.joyous.fields.MultipleSelectField(blank=True, choices=[('L', 'List View'), ('W', 'Weekly View'), ('M', 'Monthly View')], default=['L', 'W', 'M']),
        ),
    ]
