{% extends "joyous/joyous_base.html" %}
{% load wagtailcore_tags wagtailimages_tags i18n %}

{% block content %}
<div class="content">
  <div class="event clearfix">
    {% block event_status %}
    {% if page.status %}
    <div class="event-status">
      {{ page.status_text }}
    </div>
    {% endif %}
    {% endblock event_status %}
    {% block event_title %}
    <div class="event-heading">
      <h1>{{ page.title }}</h1>
    </div>
    {% endblock event_title %}
    {% block event_image %}
    <div class="event-image">
      {% image page.image width-350 %}
    </div>
    {% endblock event_image %}
    {% block event_who %}
    {% include "joyous/includes/who.html" with event=page %}
    {% endblock event_who %}
    {% block event_when %}
    {% include "joyous/includes/when.html" with event=page %}
    {% endblock event_when %}
    {% block event_where %}
    {% include "joyous/includes/where.html" with event=page %}
    {% endblock event_where %}
    {% block event_details %}
    <div class="event-details">
      {{ page.details|richtext }}
    </div> 
    {% endblock event_details %}
  </div>
  {% block event_footer %}
  <div class="event-export">
    <a href="{{ request.path }}?format=google" target="_blank">{% trans "Google Calendar" %}</a>
    &nbsp;&nbsp;&nbsp;
    <a href="{{ request.path }}?format=ical">{% trans "Export iCal" %}</a>
  </div>
  {% endblock event_footer %}
</div>
{% endblock %}
