{% extends "joyous/joyous_base.html" %}
{% load wagtailcore_tags joyous_tags %}

{% block content %}
<div class="content">
  <div class="page-heading">
    <h2>{{ page.title }}</h2>
  </div>
  <div class="content-inner">
    {{ page.content|richtext }}
    {% group_upcoming_events %}
  </div>
</div>
{% endblock %}
