{% load wagtailcore_tags wagtailimages_tags %}

<div class="event-item clearfix">
  <a class="event-title" href="{{ url }}">{{ title }}</a>
  {% if event.image %}
  <div class="event-image">
      <a class="img-link" href="{{ url }}">{% image event.image width-250 %}</a>
  </div>
  {% endif %}
  {% include "joyous/includes/who.html" %}
  {% include "joyous/includes/when.html" %}
  {% include "joyous/includes/where.html" %}
  <div class="event-details">
    {{ event.details|richtext|truncatewords_html:150 }}
  </div>
</div>   
