{% extends "joyous/exception_base.html" %}
{% load wagtailcore_tags %}

{% block event_title %}
{% if page.cancellation_title %}
  <h1 class="event-heading">{{ page.cancellation_title }}</h1>
{% else %}
  <h1 class="event-heading">{{ overrides.title }}</h1>
{% endif %}
{% endblock event_title %}

{% block event_details %}
<div class="event-cancellation-details">
  {{ page.cancellation_details|richtext }}
</div>
{% endblock event_details %}
