{% extends "joyous/exception_base.html" %}
{% load wagtailimages_tags %}

{% block event_title %}
<div class="event-heading">
  {# postponement_title is required #}
  <h1>{{ page.postponement_title }}</h1>
</div>
{% endblock %}

{% block event_image %}
<div class="event-image">
  {% image page.image width-350 %}
</div>
{% endblock event_image %}
