{% load wagtailcore_tags i18n %}
<div class="event-overrides event-index">
  <a class="item" href="{% pageurl overrides %}">{% trans "See also <strong>other occurrences</strong> of this event." %}</a>
</div>
