{% load wagtailcore_tags %}
{% if evod %}
<td class="{{ evod.weekday }} day{% if evod.date == today %} today{% endif %}">
  <div {% if evod.holiday %}class="day-title holiday" title="{{ evod.holiday }}"{% else %}class="day-title"{% endif %}>
    {% if evod.all_events %}
    {% if calendarUrl %}
    <a class="event" title="{{ evod.preview }}" href="{{ calendarUrl }}{{ evod.date|date:"Y/m/d/" }}">{{ evod.date.day }}</a>
    {% else %}
    <span class="event" title="{{ evod.preview }}">{{ evod.date.day }}</span>
    {% endif %}
    {% else %}
    <span>{{ evod.date.day }}</span>
    {% endif %}
  </div>
</td>
{% else %}
<td class="noday">&nbsp;</td>
{% endif %}
